@@ -43,12 +43,14 @@ static void activate_pause(int autoresume);
 
 // Input states for keyboard, accelerometer and touch screen
 static int keybd_keystate = 0;
+static int combo_keystate = 0;
 static int accel_keystate = 0;
 static int touch_keystate = 0;
 static int touch_idle = TRUE;
 
 // Map hardware key codes to local key numbers
 static unsigned char keymap[65536];
+static int use_zoom_keys = FALSE;
 
 // Accelerometer parameters
 static int accel_enable = FALSE;
@@ -296,6 +298,7 @@ static void activate_pause(int automatic)
 static void
 keybd_handler(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
+  int *target_keystate = &keybd_keystate;
   u32 key = keymap[event->hardware_keycode];
 
   if (key == 0xff) return;
@@ -304,25 +307,24 @@ keybd_handler(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 
   switch (key) {
     case GKEY_LEFT_UP:
-      key = GKEY_LEFT  | GKEY_UP;   break;
     case GKEY_RIGHT_UP:
-      key = GKEY_RIGHT | GKEY_UP;   break;
     case GKEY_LEFT_DOWN:
-      key = GKEY_LEFT  | GKEY_DOWN; break;
     case GKEY_RIGHT_DOWN:
-      key = GKEY_RIGHT | GKEY_DOWN; break;
+      target_keystate = &combo_keystate;
+      break;
     case GKEY_QUIT:
-      quit(); break;
+      quit();
+      return;
     case GKEY_PAUSE:
       pause_key_down = TRUE;
       activate_pause(FALSE);
-      break;
+      return;
   }
 
   if (event->type == GDK_KEY_PRESS)
-    keybd_keystate |= key;
+    *target_keystate |= key;
   else if (event->type == GDK_KEY_RELEASE)
-    keybd_keystate &= ~key;
+    *target_keystate &= ~key;
 }
 
 static void
@@ -414,6 +416,10 @@ void read_keys(FILE *keys_file)
   for (i = 0; i < 20; i++) {
     fscanf(keys_file, "%i", &key_code);
     keymap[key_code] = i;
+
+    // Check whether zoom keys should be captured
+    if (key_code == 73 || key_code == 74)
+        use_zoom_keys = TRUE;
   }
 }
 
@@ -587,9 +593,10 @@ void gpsp_plat_init(void)
   else
     fprintf(stderr, "ERROR: could not open keys configuration file\n");
 
-  // Create the main window and declare support for portrait mode
+  // Create the main window, declare support for portrait mode and zoom keys
   window = hildon_window_new ();
   hildon_gtk_window_set_portrait_flags (GTK_WINDOW(window), HILDON_PORTRAIT_MODE_SUPPORT);
+  hildon_gtk_window_enable_zoom_keys (GTK_WINDOW(window), use_zoom_keys);
 
   // Make the window black
   GdkColor window_bg = {0, 0, 0, 0};
@@ -651,7 +658,19 @@ void gpsp_plat_quit(void)
 
 inline u32 gpsp_plat_joystick_read(void)
 {
-  return keybd_keystate | accel_keystate | touch_keystate;
+  int keystate = keybd_keystate | accel_keystate | touch_keystate;
+
+  // Expand keys that translate to multiple basic keys
+  if (combo_keystate & GKEY_LEFT_UP)
+    keystate |= GKEY_LEFT | GKEY_UP;
+  if (combo_keystate & GKEY_RIGHT_UP)
+    keystate |= GKEY_RIGHT | GKEY_UP;
+  if (combo_keystate & GKEY_LEFT_DOWN)
+    keystate |= GKEY_LEFT | GKEY_DOWN;
+  if (combo_keystate & GKEY_RIGHT_DOWN)
+    keystate |= GKEY_RIGHT | GKEY_DOWN;
+
+  return keystate;
 }
 
 inline u32 gpsp_plat_buttons_to_cursor(u32 buttons)
