@@ -133,7 +133,7 @@ void trigger_ext_event();
 
 static const char *file_ext[] = { ".gba", ".bin", ".zip", NULL };
 
-#ifndef PSP_BUILD
+#if !defined(PSP_BILD) && !defined(MAEMO_BUILD)
 static void ChangeWorkingDirectory(char *exe)
 {
 #ifndef _WIN32_WCE
@@ -150,7 +150,7 @@ static void switch_to_romdir(void)
 {
   char buff[256];
   int r;
-  
+
   file_open(romdir_file, "romdir.txt", read);
 
   if(file_check_valid(romdir_file))
@@ -243,11 +243,11 @@ int main(int argc, char *argv[])
 #endif
 
 #ifndef PC_BUILD
-#ifdef MAEMO
- return maemo_init(&argc, argv);
-#else
- gpsp_plat_init();
+#ifdef MAEMO_BUILD
+  // Parse command line arguments
+  if (gpsp_plat_args(&argc, argv) != 0) exit(-1);
 #endif
+  gpsp_plat_init();
 #endif
   load_config_file();
 
@@ -257,8 +257,23 @@ int main(int argc, char *argv[])
 
   sprintf(bios_filename, "%s" PATH_SEPARATOR "%s", main_path, "gba_bios.bin");
   ret = load_bios(bios_filename);
+#ifdef MAEMO_BUILD
+  if (ret != 0) {
+    // Look in the installation directory
+    sprintf(bios_filename, "%s" PATH_SEPARATOR "%s", INSTALL_PATH, "gba_bios.bin");
+    ret = load_bios(bios_filename);
+  }
+  if (ret != 0 && argc > 1) {
+    // Look in the current ROM's directory
+    strcpy(bios_filename, argv[1]);
+    char *slash = strrchr(bios_filename, '/');
+    strcpy(slash ? slash+1 : bios_filename, "gba_bios.bin");
+    ret = load_bios(bios_filename);
+  }
+#else
   if (ret != 0)
     ret = load_bios("gba_bios.bin");
+#endif
   if (ret != 0)
   {
     gui_action_type gui_action = CURSOR_NONE;
@@ -273,7 +288,12 @@ int main(int argc, char *argv[])
     debug_screen_printl("                                                  ");
     debug_screen_printl("When you do get it name it gba_bios.bin and put it");
 #ifdef PND_BUILD
+  #ifdef MAEMO_BUILD
+    debug_screen_printl("in /home/user/.config/gpspm/, /opt/gpspm/ or ROM's");
+    debug_screen_printl("directory. Locations are checked in that order.   ");
+  #else
     debug_screen_printl("in <SD card>/pandora/appdata/gpsp/ .              ");
+  #endif
 #else
     debug_screen_printl("in the same directory as gpSP.                    ");
 #endif
@@ -991,6 +1011,13 @@ void change_ext(const char *src, char *buffer, const char *extension)
     strcpy(dot_position, extension);
 }
 
+#ifdef MAEMO_BUILD
+// make path: <romdir>/<romname>.<ext>
+void make_rpath(char *buff, size_t size, const char *ext)
+{
+  change_ext(gamepak_filename, buff, ext);
+}
+#else
 // make path: <main_path>/<romname>.<ext>
 void make_rpath(char *buff, size_t size, const char *ext)
 {
@@ -1004,6 +1031,7 @@ void make_rpath(char *buff, size_t size, const char *ext)
   if (p != NULL)
     strcpy(p, ext);
 }
+#endif
 
 #define main_savestate_builder(type)                                          \
 void main_##type##_savestate(file_tag_type savestate_file)                    \
